/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.03.09.
*   Chip         -  Atmel ATmega8
*   Compiler     -  avr-gcc (WinAVR 20080610) 4.3.0
*
*   Definitions for ATMega8
*
********************************************************************************
*
*                              +---------+
*               NRESET - PC6  ++ 1    28 |  PC5 - ADC5 - SCL
*                  RXD - PD0  |  2    27 |  PC4 - ADC4 - SDA
*                  TXD - PD1  |  3    26 |  PC3 - ADC3
*                 INT0 - PD2  |  4    25 |  PC2 - ADC2
*                 INT1 - PD3  |  5    24 |  PC1 - ADC1
*             XCK - T0 - PD4  |  6    23 |  PC0 - ADC0
*                        VCC  |  7    22 |  GND
*                        GND  |  8    21 |  AREF
*         XTAL1 - OSC1 - PB6  |  9    20 |  AVCC
*         XTAL2 - OSC2 - PB7  | 10    19 |  PB5 - SCK
*                   T1 - PD5  | 11    18 |  PB4 - MISO
*                 AIN0 - PD6  | 12    17 |  PB3 - MOSI - OC2
*                 AIN1 - PD7  | 13    16 |  PB2 - NSS - OC1B
*                 ICP1 - PB0  | 14    15 |  PB1 - OC1A
*                             +----------+
*
********************************************************************************
*
*              32       25
*           1  +---------+ 24
*             ++         |
*             |          |
*             |          |
*             |          |
*             |          |
*           8 +----------+ 17
*             9         16
*
*    1 - PD3 - INT1
*    2 - PD4 - T0    - XCK
*    3 - GND
*    4 - VCC
*    5 - GND
*    6 - VCC
*    7 - PB6 - TOSC1 - XTAL1
*    8 - PB7 - TOSC2 - XTAL2
*
*    9 - PD5 - T1
*   10 - PD6 - AIN0
*   11 - PD7 - AIN1
*   12 - PB0 - ICP1
*   13 - PB1 - OC1A
*   14 - PB2 - OC1B  - SS
*   15 - PB3 - MOSI  - OC2
*   16 - PB4 - MISO
*
*   17 - PB5 - SCK
*   18 - AVCC
*   19 - ADC8
*   20 - AREF
*   21 - GND
*   22 - ADC7
*   23 - PC0 - ADC0
*   24 - PC1 - ADC1
*
*   25 - PC2 - ADC2
*   26 - PC3 - ADC3
*   27 - PC4 - ADC4  - SDA
*   28 - PC5 - ADC5  - SCL
*   29 - PC6 - NRESET
*   30 - PD0 - RXD
*   31 - PD1 - TXD
*   32 - PD2 - INT0
*
********************************************************************************
*   PonyProg Configuration and Security Bits (P - bepipalva):
*
*   BootLock12        - Boot Lock Bit1
*   BootLock11
*   BootLock02        - Boot Lock Bit0
*   BootLock02
*   LOCK2             - Memory protection
*   LOCK1
*
*   RSTDISBLE         - Select if PC6 is I/O pin or RESET pin
*   WDTON             - Watchdog Timer always on
*   SPIEN             - Enable Serial Program and Data Downloading
*   CKOPT             - Oscillator options
*   EESAVE            - EEPROM memory is preserved through the Chip Erase
*   BOOTESZ1          - Select Boot Size
*   BOOTESZ0
*   BOOTRST           - Select Reset Vector
*
*   BODLEVEL          - 2.7V / 4V (P)
*   BODEN             - Brown-out detection disable / enable (P)
*   SUT1              - Select start-up time
*   SUT0
*   CKSEL3            - Select Clock source
*   CKSEL2
*   CKSEL1
*   CKSEL0
*
*   PonyProg Configuration and Security Bits (bepipalva):
*
*   CKSEL3, CKSEL2, CKSEL1, CKSEL0
*   P       P       P       P       0000  External Clock
*
*   P       P       P       -       0001  Calibrated Internal RC Oscillator 1MHz
*   P       P       -       P       0010  Calibrated Internal RC Oscillator 2MHz
*   P       P       -       -       0011  Calibrated Internal RC Oscillator 4MHz
*   P       -       P       P       0100  Calibrated Internal RC Oscillator 8MHz
*   P       -       P       -       0101  External RC Oscillator 0.1-0.9MHz
*   P       -       -       P       0110  External RC Oscillator 0.9-3MHz
*   P       -       -       -       0111  External RC Oscillator 3-8MHz
*   -       P       P       P       1000  External RC Oscillator 8MHz-
*   -       P       P       -       1001  External Low-frequency Crystal
*   -       P       -       P       1010  Ceramic resonator 0.4-0.9MHz
*   -       P       -       -       1011  Crystal Oscillator 0.4-0.9MHz
*   -       -       P       P       1100  Ceramic resonator 0.9-3MHz
*   -       -       P       -       1101  Crystal Oscillator 0.9-3MHz
*   -       -       -       P       1110  Ceramic resonator 3-8MHz
*   -       -       -       P       1110  Ceramic resonator 8MHz-, +CKOPT P
*   -       -       -       -       1111  Crystal Oscillator 3-8MHz
*   -       -       -       -       1111  Crystal Oscillator 8MHz-, +CKOPT P
*
*   Calibrated Internal RC Oscillator
*   SUT1   SUT0
*   P      P        00  BOD enabled
*   P      -        01  Fast rising power
*   -      P        10  Slowly rising power
*   -      -        11  Reserved
*
*   Ceramic resonator
*   SUT1   SUT0
*   P      P        00  Fast rising power
*   P      -        01  Slowly rising power
*   -      P        10  BOD enabled
*   -      -        11  Fast rising power

*   External Crystal
*   SUT1   SUT0
*   P      P        00  Slowly rising power
*   P      -        01  BOD enabled
*   -      P        10  Fast rising power
*   -      -        11  Slowly rising power
*
*   LOCK2   LOCK1
*   -       -       11  No memory lock
*   -       P       10  Programming disable
*   P       P       11  Programming and verification disable
*
*   Boot loader size
*   BOOTESZ1  BOOTESZ0
*   -         -         11   128 world
*   -         P         10   256 world
*   P         -         01   512 world
*   P         P         00  1024 world
*
*******************************************************************************/

#include <avr/sfr_defs.h>
#include "tkiraaly_avr_macros.h"


#define TWBR      _SFR_IO8( 0x00)                // TWI Bit Rate Register

#define TWSR      _SFR_IO8( 0x01)                // TWI Status Register
#define    TWS7      7                           // TWI Status 7..3
#define    TWS6      6
#define    TWS5      5
#define    TWS4      4
#define    TWS3      3
#define    TWPS1     1                           // TWI Prescaler Bits 1..0
#define    TWPS0     0
//  TWPS1..0
//      0  0 -  1
//      0  1 -  4
//      1  0 - 16
//      1  1 - 64

#define TWAR      _SFR_IO8( 0x02)                // TWI (Slave) Address Register
#define    TWA6      7                           // TWI (Slave) Address 6..0
#define    TWA5      6
#define    TWA4      5
#define    TWA3      4
#define    TWA2      3
#define    TWA1      2
#define    TWA0      1
#define    TWGCE	   0                           // TWI General Call Recognition Enable Bit

#define TWDR      _SFR_IO8( 0x03)

#define ADC       _SFR_IO16( 0x04)               // ADC Data Register
#define ADCL      _SFR_IO8( 0x04)
#define ADCH      _SFR_IO8( 0x05)

#define ADCSRA    _SFR_IO8(0x06)                 // ADC Control and Status Register A
#define    ADEN      7                           // ADC Enable
#define    ADSC      6                           // ADC Start Conversion
#define    ADATE     5                           // ADC Auto Trigger Enable
#define    ADIF      4                           // ADC Interrupt Flag
#define    ADIE      3                           // ADC Interrupt Enable
#define    ADPS2     2                           // ADC Prescaler Select Bits
#define    ADPS1     1                           // ADC Prescaler Select Bits
#define    ADPS0     0                           // ADC Prescaler Select Bits
// Div  ADPS3..0
//   2  000
//   2  001
//   4  010
//   8  011
//  16  100
//  32  101
//  64  110
// 128  111

#define ADMUX     _SFR_IO8(0x07)                 // ADC Multiplexer Selection Register
#define    REFS1     7                           // Reference Selection Bit 1..0
#define    REFS0     6                           // Analog Channel Selection Bits 3..0
#define    ADLAR     5                           // ADC Left Adjust Result
#define    MUX3      3
#define    MUX2      2
#define    MUX1      1
#define    MUX0      0
//  MUX3..0
//     0000 - ADC0
//     0001 - ADC1
//     0010 - ADC2
//     0011 - ADC3
//     0100 - ADC4
//     0101 - ADC5
//     0110 - ADC6
//     0111 - ADC7
//     0000 - ADC0
//     0000 - ADC0
//     1110 - 1.30V (V BG)
//     1111 - 0V (GND)

// REFS1..0
//     0  0 - AREF, Internal V ref turned off
//     0  1 - AV CC with external capacitor at AREF pin
//     1  1 - Internal 2.56V Voltage Reference with external capacitor at AREF pin

#define ACSR      _SFR_IO8(0x08)                 // Analog Comparator Control and Status Register
#define    ACD       7                           // Analog Comparator Disable
#define    ACBG      6                           // Analog Comparator Bandgap Select
#define    ACO       5                           // Analog Comparator Output
#define    ACI       4                           // Analog Comparator Interrupt Flag
#define    ACIE      3                           // Analog Comparator Interrupt Enable
#define    ACIC      2                           // Analog Comparator Input Capture Enable
#define    ACIS1     1                           // Analog Comparator Interrupt Mode Select 1..0
#define    ACIS0     0
// ACIS1  ACIS0
//     0      0      - Comparator Interrupt on Output Toggle
//     1      0      - Comparator Interrupt on Falling Output Edge
//     1      1      - Comparator Interrupt on Rising Output Edge

#define UBRRL     _SFR_IO8( 0x09)                // USART Baud Rate Register L

#define UCSRB     _SFR_IO8( 0x0A)                // USART Control and Status Register B
#define    RXCIE     7                           // RX Complete Interrupt Enable
#define    TXCIE     6                           // TX Complete Interrupt Enable
#define    UDRIE     5                           // USART Data Register Empty Interrupt Enable
#define    RXEN      4                           // Receive Enable
#define    TXEN      3                           // Transmitter Enable
#define    UCSZ2     2                           // Character Size
#define    RXB8      1                           // Receive Data Bit 8
#define    TXB8      0                           // Transmit Data Bit 8

#define UCSRA     _SFR_IO8( 0x0B)                // USART Control and Status Register A
#define    RXC       7                           // USART Receive Complete
#define    TXC       6                           // USART Transmit Complete
#define    UDRE      5                           // USART Data Register Empty
#define    FE        4                           // Frame Error
#define    DOR       3                           // Data Overrun
#define    PE        2                           // USART Parity Error
#define    U2X       1                           // Double the USART Transmission Speed
#define    MPCM      0                           // Multiprocessor Communication Mode

#define UDR       _SFR_IO8( 0x0C)                // USART I/O Data Register

#define SPCR      _SFR_IO8(0x0D)                 // SPI Control Register
#define    SPIE      7                           // SPI Interrupt Enable
#define    SPE       6                           // SPI Enable
#define    DORD      5                           // Data Order
#define    MSTR      4                           // Master/Slave Select
#define    CPOL      3                           // Clock Polarity
#define    CPHA      2                           // Clock Phase
#define    SPR1      1                           // SPI Clock Rate Select 1
#define    SPR0      0                           // SPI Clock Rate Select 0
//  SPSR  SPCR  SPCR
// SPI2X  SPR1  SPR2
//     0     0     0 - F_CPU/   4
//     0     0     1 - F_CPU/  16
//     0     1     0 - F_CPU/  64
//     0     1     1 - F_CPU/ 128
//     1     0     0 - F_CPU/   2
//     1     0     1 - F_CPU/   8
//     1     1     0 - F_CPU/  32
//     1     1     1 - F_CPU/  64

#define SPSR      _SFR_IO8(0x0E)                 // SPI Status Register
#define    SPIF      7                           // SPI Interrupt Flag
#define    WCOL      6                           // Write Collision Flag
#define    SPI2X     0                           // Double SPI Speed Bit

#define SPDR      _SFR_IO8( 0x0F)

#define PIND      _SFR_IO8( 0x10)                // Port D - Input Register

#define DDRD      _SFR_IO8( 0x11)                // Port D - Data Direction Register

#define PORTD     _SFR_IO8( 0x12)                // Port D - Output Register

#define PINC      _SFR_IO8( 0x13)                // Port C - Input Register

#define DDRC      _SFR_IO8( 0x14)                // Port C - Data Direction Register

#define PORTC     _SFR_IO8( 0x15)                // Port C - Output Register

#define PINB      _SFR_IO8( 0x16)                // Port B - Input Register

#define DDRB      _SFR_IO8( 0x17)                // Port B - Data Direction Register

#define PORTB     _SFR_IO8( 0x18)                // Port B - Output Register

#define EECR      _SFR_IO8( 0x1C)                // EEPROM Control Register
#define    EERIE     3                           // EEPROM Ready Interrupt Enabla Bit
#define    EEMPE     2                           // EEPROM Master Program Enable Bit
#define    EEPE      1                           // EEPROM Program Enable Bit
#define    EERE      0                           // EEPROM Read Enable Bit

#define EEDR      _SFR_IO8( 0x1D)                // EEPROM Data Register

#define EEAR      _SFR_IO16( 0x1E)               // EEPROM Address Register
#define EEARL     _SFR_IO8( 0x1E)
#define EEARH     _SFR_IO8( 0x1F)

#define UCSRC     _SFR_MEM8(0x20)                // USART Control and Status Register C
#define    URSEL     7                           // USART Register Select - 0
#define    UMSEL     6                           // USART Mode Select
#define    UPM1      5                           // USART Parity Mode 1
#define    UPM0      4                           // USART Parity Mode 0
#define    USBS      3                           // USART Stop Bit Select
#define    UCSZ1     2                           // USART Character Size 1
#define    UCSZ0     1                           // USART Character Size 0
#define    UCPOL     0                           // USART Clock Polarity

#define UBRRH   _SFR_IO8( 0x20)                  // USART Baud Rate Register H
#define    URSEL     7                           // USART Register Select - 1
#define    UBRB11    3                           // USART Baud Rate Bits - 11..8
#define    UBRB10    2
#define    UBRB9     1
#define    UBRB8     0

#define WDTCR     _SFR_IO8(0x21)                 // Watchdog Timer Control Register
#define    WDCE      4                           // Watchdog Change Enable
#define    WDE       3                           // Watchdog Enable
#define    WDP2      2                           // Watchdog Timer Prescaler 2..0
#define    WDP1      1
#define    WDP0      0
// WDP2..0                 cycles     ~ time out
//     000           -        16k        17 msec
//     001           -        32k        34 msec
//     010           -        64k        68 msec
//     011           -       128k      0,14  sec
//     100           -       256k      0,27  sec
//     101           -       512k      0,55  sec
//     110           -         1M      1,1   sec
//     111           -         2M      2,2   sec
#define ASSR      _SFR_IO8(0x22)                  // Asynchronous Status Register
#define    AS2       3                            // Asynchronous Timer/Counter 2
#define    TCN2UB    2                            // Timer/Counter 2 Update Busy
#define    OCR2UB    1                            // Output Compare Register 2 Update Busy
#define    TCR2UB    0                            // Timer/Counter Control Register 2 Update Busy


#define OCR2      _SFR_IO8( 0x23)                // Timer 2

#define TCNT2     _SFR_IO8( 0x24)

#define TCCR2     _SFR_IO8(0x25)                 // Timer/Counter 2 Control register
#define    FOC2      7                           // Force Output Compare
#define    WGM20     6                           // Waveform Generation Mode 0
#define    COM21     5                           // Compare Match Output Mode 1..0
#define    COM20     4
#define    WGM21     3                           // Waveform Generation Mode 1
#define    CS22      2                           // Clock Select 2..0
#define    CS21      1
#define    CS20      0
// WGM22  WGM21
//     0      0      - Normal
//     0      1      - PWM, Phase Correct
//     1      0      - CTC
//     1      1      - Fast PWM
//
//  CS22..0
//      000          - No clock source (Timer/Counter stopped)
//      001          - F_CPU/    1
//      010          - F_CPU/    8
//      011          - F_CPU/   64
//      100          - F_CPU/  256
//      101          - F_CPU/ 1024
//      110          - External clock source on T2 pin. Clock on falling edge.
//      111          - External clock source on T2 pin. Clock on falling edge.

#define ICR1	    _SFR_IO16( 0x26)               // Input Capture Register 1
#define ICR1L	    _SFR_IO8( 0x26)
#define ICR1H     _SFR_IO8( 0x27)

#define OCR1B     _SFR_IO16( 0x28)               // Output Compare Register 1 B
#define OCR1BL    _SFR_IO8( 0x28)
#define OCR1BH    _SFR_IO8( 0x29)

#define OCR1A	    _SFR_IO16( 0x2A)               // Output Compare Register 1 A
#define OCR1AL    _SFR_IO8( 0x2A)
#define OCR1AH    _SFR_IO8( 0x2B)

#define TCNT1     _SFR_IO16( 0x2C)               // Timer/Counter 1
#define TCNT1L    _SFR_IO8( 0x2C)
#define TCNT1H    _SFR_IO8( 0x2D)

#define TCCR1B    _SFR_IO8(0x2E)                 // Timer/Counter 1 Control and Status Register
#define    ICNC1     7                           // Input Capture Noise Canceler
#define    ICES1     6                           // Input Capture Edge Select
#define    WGM13     4                           // Waveform Generation Mode 3..2
#define    WGM12     3
#define    CS12      2                           // Clock Select 2..1
#define    CS11      1
#define    CS10      0

#define TCCR1A    _SFR_IO8(0x2F)                 // Timer/Counter 1 Control Register
#define    COM1A1    7                           // Compare Output Mode for Channel A 1..0
#define    COM1A0    6
#define    COM1B1    5                           // Compare Output Mode for Channel B 1..0
#define    COM1B0    4
#define    FOC1A     3                           // Force Output Compare for channel A
#define    FOC1B     2                           // Force Output Compare for channel B
#define    WGM11     1                           // Waveform Generation Mode 1..0
#define    WGM10     0

#define SFIOR     _SFR_IO8(0x30)                 // Special Function I/O Register
#define    ACME      3                           // Analog Comparator Multiplexer Enable
#define    PUD       2                           // Pull-up Disable
#define    PSR2      1                           // Prescaler Reset Timer/Counter 2
#define    PSR10     0                           // Prescaler Reset Timer/Counter 1, 0


#define OSCCAL    _SFR_IO8( 0x31)                // Oscillator Calibration Register

#define TCNT0     _SFR_IO8( 0x32)                // Timer/Counter Register 0

#define TCCR0     _SFR_IO8( 0x33)                // Timer/Counter Control Register
#define    CS02      2                           // Clock Set 2
#define    CS01      1                           // Clock Set 1
#define    CS00      0                           // Clock Set 0
//  CS02..0
//      000          - No clock source (Timer/Counter stopped)
//      001          - F_CPU/    1
//      010          - F_CPU/    8
//      011          - F_CPU/   64
//      100          - F_CPU/  256
//      101          - F_CPU/ 1024
//      110          - External clock source on T2 pin. Clock on falling edge.
//      111          - External clock source on T2 pin. Clock on falling edge.

#define MCUCSR    _SFR_IO8(0x34)                  // MCU Status Register
#define    WDRF      3                            // Watchdog Reset Flag
#define    BORF      2                            // Brown-out Reset Flag
#define    EXTRF     1                            // External Reset Flag
#define    PORF      0                            // Power-on Reset Flag

#define MCUCR     _SFR_IO8(0x35)                  // MCU general Control Register
#define    SE        7                            // Sleep Enable
#define    SM2       6                            // Sleep Mode Select Bits 2..0
#define    SM1       5
#define    SM0       4
#define    ISC11     3                            //  Interrupt Sense Control 11..00
#define    ISC10     2
#define    ISC01     1
#define    ISC00     0
//   SM2..0
//      000          - Idle
//      001          - ADC Noise Reduction
//      010          - Power-down
//      011          - Power-save
//      110          - Standby

//  ISC11..0
//      0  0         - The low level of INT1 generates an interrupt request
//      0  1         - Any logical change on INT1 generates an interrupt request
//      1  0         - The falling edge of INT1 generates an interrupt request
//      1  1         - The rising edge of INT1 generates an interrupt request

//  ISC01..0
//      0  0         - The low level of INT0 generates an interrupt request
//      0  1         - Any logical change on INT0 generates an interrupt request
//      1  0         - The falling edge of INT0 generates an interrupt request
//      1  1         - The rising edge of INT0 generates an interrupt request

#define TWCR      _SFR_IO8(0x36)                 // TWI Control Register
#define    TWINT     7                           // TWI Interrupt Flag
#define    TWEA      6                           // TWI Enable Acknowledge
#define    TWSTA     5                           // TWI START Condition
#define    TWSTO     4                           // TWI STOP Condition
#define    TWWC      3                           // TWI Write Collision Flag
#define    TWEN      2                           // TWI Enable
#define    TWIE      0                           // TWI Interrupt Enable

#define SPMCR     _SFR_IO8( 0x37)                // Store Program Memory Control Register
#define    SPMIE     7                           // SPM Interrupt Enable
#define    RWWSB     6                           // Read-While-Write Section Busy
#define    RWWSRE    4                           // Read-While-Write Section Read Enable
#define    BLBSET    3                           // Boot Lock Bit Set
#define    PGWRT     2                           // Page Write
#define    PGERS     1                           // Page Erase
#define    SPMEN     0                           // Store Program Memory Enable

#define TIFR      _SFR_IO8( 0x38)                // Timer/Counter Interrupt Flag Register
#define    OCF2      7                           // Output Compare Flag 2
#define    TOV2      6                           // Timer/Counter 2 Overflow Flag
#define    ICF1      5                           // Timer/Counter 1, Input Capture Flag
#define    OCF1A     4                           // Timer/Counter 1, Output Compare A Match Flag
#define    OCF1B     3                           // Timer/Counter 1, Output Compare B Match Flag
#define    TOV1      2                           // Timer/Counter 1, Overflow Flag
#define    TOV0      0                           // Timer/Counter 0 Overflow Flag

#define TIMSK     _SFR_IO8( 0x39)                // Timer/Counter Interrupt Mask Register
#define    OCIE2     7                           // Timer/Counter 2 Output Compare Match Interrupt Enable
#define    TOIE2     6                           // Timer/Counter 2 Overflow Interrupt Enable
#define    TICIE1    5                           // Timer/Counter 1, Input Capture Interrupt Enable
#define    OCIE1A    4                           // Timer/Counter 1, Output Compare A Match Interrupt Enable
#define    OCIE1B    3                           // Timer/Counter 1, Output Compare B Match Interrupt Enable
#define    TOIE1     2                           // Timer/Counter 1, Overflow Interrupt Enable
#define    TOIE0     0                           // Timer/Counter 0 Overflow Interrupt Enable

#define GIFR      _SFR_IO8( 0x3A)                // General Interrupt Flag Register
#define    INTF1     7                           // External Interrupt Flag 1
#define    INTF0     6                           // External Interrupt Flag 0

#define GICR	    _SFR_IO8( 0x3B)                // General Interrupt Control Register
#define    INT1      7                           // External Interrupt Request 1 Enable
#define    INT0      6                           // External Interrupt Request 0 Enable
#define    IVSEL     1                           // External Interrupt Flag 1
#define    IVCE      0                           // External Interrupt Flag 0

/* Constants */
#define _VECTORS_SIZE 38
#define SPM_PAGESIZE    64
#define RAMEND       0x45F
#define XRAMEND     RAMEND
#define E2END        0x1FF
#define E2PAGESIZE       4
#define FLASHEND    0x1FFF

/* Fuses */
#define FUSE_MEMORY_SIZE 2

/* Low Fuse Byte */
#define FUSE_CKSEL0      (unsigned char)~_BV(0)
#define FUSE_CKSEL1      (unsigned char)~_BV(1)
#define FUSE_CKSEL2      (unsigned char)~_BV(2)
#define FUSE_CKSEL3      (unsigned char)~_BV(3)
#define FUSE_SUT0        (unsigned char)~_BV(4)
#define FUSE_SUT1        (unsigned char)~_BV(5)
#define FUSE_BODEN       (unsigned char)~_BV(6)
#define FUSE_BODLEVEL    (unsigned char)~_BV(7)
#define LFUSE_DEFAULT (FUSE_SUT0 & FUSE_CKSEL3 & FUSE_CKSEL2 & FUSE_CKSEL1)

/* High Fuse Byte */
#define FUSE_BOOTRST     (unsigned char)~_BV(0)
#define FUSE_BOOTSZ0     (unsigned char)~_BV(1)
#define FUSE_BOOTSZ1     (unsigned char)~_BV(2)
#define FUSE_EESAVE      (unsigned char)~_BV(3)
#define FUSE_CKOPT       (unsigned char)~_BV(4)
#define FUSE_SPIEN       (unsigned char)~_BV(5)
#define FUSE_WDTON       (unsigned char)~_BV(6)
#define FUSE_RSTDISBL    (unsigned char)~_BV(7)
#define HFUSE_DEFAULT (FUSE_SPIEN & FUSE_BOOTSZ1 & FUSE_BOOTSZ0)

/* Lock Bits */
#define __LOCK_BITS_EXIST
#define __BOOT_LOCK_BITS_0_EXIST
#define __BOOT_LOCK_BITS_1_EXIST

/* Signature */
#define SIGNATURE_0 0x1E
#define SIGNATURE_1 0x93
#define SIGNATURE_2 0x07




// KT macro-k

#define U8                   unsigned char
#define U16                  unsigned int

#define KHZ                  *1000
#define MHZ                  *1000000

#define OCR_TIMER( ido_usec, osztas)  ido_usec* FCPU/ osztas/ 1000000UL+ 1UL
#define OCR_CLOCK( f_hz, osztas)      F_CPU/ osztas/ f_hz- 1UL
#define OCR_WAVE( f_hz, osztas)       F_CPU/ 2UL/ osztas/ f_hz- 1UL

#define B_1(sfr,bit)         sfr|= ( 1 << ( bit))               // egy regiszter adott bitjenek 1-re allitas
#define B_0(sfr,bit)         sfr&= ~( 1 << ( bit))              // egy regiszter adott bitjenek 0-ra allitas
#define B_R(sfr,bit)         bit_is_set( sfr, bit)              // egy regiszter egy bitjenek olvasasa

#define NOP                  __asm__ __volatile__ ("nop" ::)    // egy ures utasitas beszurasa




// Interrupts

#define IT_ENABLE            __asm__ __volatile__ ("sei" ::)    // megszakitas engedelyezese
#define IT_DISABLE           __asm__ __volatile__ ("cli" ::)    // megszakitas tiltasa

#define IT_INT0            _VECTOR( 1)                          // External Interrupt Request 0
#define IT_INT0_ENABLE           B_1(GICR,INT0)
#define IT_INT0_DISABLE          B_0(GICR,INT0)
#define IT_INT0_LEVEL_0          MCUCR&= 0B11111100
#define IT_INT0_ALL_EDGES        MCUCR&= 0B11111100; MCUCR|= 0B00000001
#define IT_INT0_FALLING          MCUCR&= 0B11111100; MCUCR|= 0B00000010
#define IT_INT0_RISING                               MCUCR|= 0B00000011

#define IT_INT1            _VECTOR( 2)                          // External Interrupt Request 1
#define IT_INT1_ENABLE           B_1(GICR,INT1)
#define IT_INT1_DISABLE          B_0(GICR,INT1)
#define IT_INT1_LEVEL_0          MCUCR&= 0B11110011
#define IT_INT1_ALL_EDGES        MCUCR&= 0B11110011; MCUCR|= 0B00000100
#define IT_INT1_FALLING          MCUCR&= 0B11110011; MCUCR|= 0B00001000
#define IT_INT1_RISING                               MCUCR|= 0B00001100

#define IT_TC2_COMP        _VECTOR( 3)                          // Timer/Counter2 Compare Match
#define IT_TC2_COMP_ENABLE    B_1(TIMSK,OCIE2)
#define IT_TC2_COMP_DISABLE   B_0(TIMSK,OCIE2)

#define IT_TC2_OVF         _VECTOR( 4)                          // Timer/Counter2 Overflow
#define IT_TC2_OVF_ENABLE     B_1(TIMSK,TOIE2)
#define IT_TC2_OVF_DISABLE    B_0(TIMSK,TOIE2)

#define IT_TC1_CAPT        _VECTOR( 5)                          // Timer/Counter1 Capture Event
#define IT_TC1_CAPT_ENABLE    B_1(TIMSK,TICIE1)
#define IT_TC1_CAPT_DISABLE   B_0(TIMSK,TICIE1)

#define IT_TC1_COMPA       _VECTOR( 6)                          // Timer/Counter1 Compare Match A
#define IT_TC1_COMPA_ENABLE   B_1(TIMSK,OCIE1A)
#define IT_TC1_COMPA_DISABLE  B_0(TIMSK,OCIE1A)

#define IT_TC1_COMPB       _VECTOR( 7)                          // Timer/Counter1 Compare Match B
#define IT_TC1_COMPB_ENABLE   B_1(TIMSK,OCIE1B)
#define IT_TC1_COMPB_DISABLE  B_0(TIMSK,OCIE1B)

#define IT_TC1_OVF         _VECTOR( 8)                          // Timer/Counter1 Overflow
#define IT_TC1_OVF_ENABLE     B_1(TIMSK,TOIE1)
#define IT_TC1_OVF_DISABLE    B_0(TIMSK,TOIE1)

#define IT_TC0_OVF	       _VECTOR( 9)                          // Timer/Counter0 Overflow
#define IT_TC0_OVF_ENABLE     B_1(TIMSK,TOIE0)
#define IT_TC0_OVF_DISABLE    B_0(TIMSK,TOIE0)

#define IT_SPI_STC			   _VECTOR( 10)                         // Serial Transfer Complete
#define IT_SPI_STC_ENABLE        B_1(SPCR,SPIE)
#define IT_SPI_STC_DISABLE       B_0(SPCR,SPIE)

#define IT_USART_RXC		   _VECTOR( 11)                         // USART, Rx Complete
#define IT_USART_RXC_ENABLE      B_1(UCSRB,RXCIE)
#define IT_USART_RXC_DISABLE     B_0(UCSRB,RXCIE)

#define IT_USART_UDRE		   _VECTOR( 12)                         // USART Data Register Empty
#define IT_USART_UDRE_ENABLE     B_1(UCSRB,UDRIE)
#define IT_USART_UDRE_DISABLE    B_0(UCSRB,UDRIE)

#define IT_USART_TXC		   _VECTOR( 13)                         // USART, Tx Complete
#define IT_USART_TXC_ENABLE      B_1(UCSRB,TXCIE)
#define IT_USART_TXC_DISABLE     B_0(UCSRB,TXCIE)

#define IT_ADC			       _VECTOR( 14)                         // ADC Conversion Complete
#define IT_ADC_ENABLE            B_1(ADCSRA,ADIE)
#define IT_ADC_DISABLE           B_0(ADCSRA,ADIE)

#define IT_EE_RDY			     _VECTOR( 15)                         // EEPROM Ready
#define IT_EE_RDY_ENABLE         B_1(EECR,EERIE)
#define IT_EE_RDY_DISABLE        B_0(EECR,EERIE)

#define IT_ACOMP			     _VECTOR( 16)                         // Analog Comparator
#define IT_ACOMP_ENABLE       B_1(ACSR,ACIE)
#define IT_ACOMP_DISABLE      B_0(ACSR,ACIE)

#define IT_TWI			       _VECTOR( 17)                         // 2-wire Serial Interface
#define IT_TWI_ENABLE            B_1(TWCR,TWIE)
#define IT_TWI_DISABLE           B_0(TWCR,TWIE)

#define IT_SPM_RDY  			 _VECTOR( 18)                         // Store Program Memory Ready
#define IT_SPM_RDY_ENABLE        B_1(SPMCR,SPMIE)
#define IT_SPM_RDY_DISABLE       B_0(SPMCR,SPMIE)

#define ISR(vector) void vector (void) __attribute__ ((signal,used)); void vector (void)




// USART-hoz
// UDR, UCSRA, UCSRB, UCSRC, UBRL, UBRH

#define KTUBRR               F_CPU/ 16UL/ BAUDRATE - 1

#define USART_UCSRC          0B10000000          // UCSRC kivalasztasa

#define USART_ASYNCRON       0                   // asszinkron mukodes     - UCSR_C
#define USART_SYNCRON        0B01000000          // szinkron mukodes       - UCSR_C

#define USART_RX_ENABLE      0B00010000          // vetel engedelyezese    - UCSR_B
#define USART_TX_ENABLE      0B00001000          // adas engedelyezese     - UCSR_B

#define USART_NO_PARITY      0                   // nincs paritas bit      - UCSR_C
#define USART_EVEN_PARITY    0B00100000          // paros paritas          - UCSR_C
#define USART_ODD_PARITY     0B00110000          // paratlan paritas       - UCSR_C

#define USART_1_STOPBIT      0                   // 1 stopbit              - UCSR_C
#define USART_2_STOPBITS     0B00001000          // 2 stopbit              - UCSR_C

#define USART_7_BIT          0B00000100          // 7 bit-es karakterhossz - UCSR_C
#define USART_8_BIT          0B00000110          // 8 bit-es karakterhossz - UCSR_C

#define USART_9600_8N1       UBRRL= F_CPU/ 16UL/ 9600- 1; \
                             UBRRH= ( F_CPU/ 16UL/ 9600- 1)>> 8; \
                             UCSRC= USART_UCSRC+ USART_ASYNCRON+ USART_8_BIT+ USART_NO_PARITY+ USART_1_STOPBIT; \
                             UCSRB= RX_ENABLE+ TX_ENABLE; \
                             IT_USART_RX_ENABLE

#define USART_19200_8N1      UBRRL= F_CPU/ 16UL/ 19200- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 19200- 1)>> 8;  \
                             UCSRC= USART_UCSRC+ USART_ASYNCRON+ USART_8_BIT+ USART_NO_PARITY+ USART_1_STOPBIT; \
                             UCSRB= RX_ENABLE+ TX_ENABLE; \
                             IT_USART_RX_ENABLE

#define USART_115200_8N1     UBRRL= F_CPU/ 16UL/ 115200- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 115200- 1)>> 8;  \
                             UCSRC= USART_UCSRC+ USART_ASYNCRON+ USART_8_BIT+ USART_NO_PARITY+ USART_1_STOPBIT; \
                             UCSRB= RX_ENABLE+ TX_ENABLE; \
                             IT_USART_RX_ENABLE




// ADC-hez

// ADC orajel elooszto valaszasa, lehetoleg nagy orajelhez
#ifndef ADC_PRESC
   #if F_CPU > 12 MHZ                      // div 128
      #define                ADC_PRESC  0B00000111
   #else
      #if F_CPU > 6 MHZ                    // div 64
         #define             ADC_PRESC  0B00000110
      #else
         #if F_CPU > 3 MHZ                 // div 32
            #define          ADC_PRESC  0B00000101
         #else
            #if F_CPU > 1,5 MHZ             // div 16
               #define       ADC_PRESC  0B00000100
            #else
               #if F_CPU > 0,75 MHZ         // div 8
                  #define    ADC_PRESC  0B00000011
               #else
                  #if F_CPU > 0,37 MHZ      // div 4
                     #define ADC_PRESC  0B00000010
                  #else                     // div 2
                     #define ADC_PRESC  0B00000001
                  #endif
               #endif
            #endif
         #endif
      #endif
   #endif
#endif

#define ADC_8BIT             B_1( ADMUX, ADLAR)                      // 8 bit felbontas, ADCH
#define ADC_10BIT            B_0( ADMUX, ADLAR)                      // 10 bit felbontas, ADCH+ ADCL
#define ADC_VREF_AREF        ADMUX&= 0B00111111                      // Uref - AREF lab
#define ADC_VREF_AVCC        ADMUX&= 0B00111111; ADMUX|= 0B01000000  // Uref - AVCC lab (analog tap))
#define ADC_VREF_INT                             ADMUX|= 0B11000000  // Uref - belso 2V56

#define ADC_IN0              ADMUX&= 0B11110000                      // bemenet valasztas
#define ADC_IN1              ADMUX&= 0B11110000; ADMUX|= 0B00000001
#define ADC_IN2              ADMUX&= 0B11110000; ADMUX|= 0B00000010
#define ADC_IN3              ADMUX&= 0B11110000; ADMUX|= 0B00000011
#define ADC_IN4              ADMUX&= 0B11110000; ADMUX|= 0B00000100
#define ADC_IN5              ADMUX&= 0B11110000; ADMUX|= 0B00000101
#define ADC_IN6              ADMUX&= 0B11110000; ADMUX|= 0B00000110
#define ADC_IN7              ADMUX&= 0B11110000; ADMUX|= 0B00000111
#define ADC_INT_1V3          ADMUX&= 0B11110000; ADMUX|= 0B00001110  // belso ref.
#define ADC_INT_0V                               ADMUX|= 0B00001111  // belso ref.

#define ADC_FREE_RUN         ADCSRA= 0B11100000+ ADC_PRESC           // ADC folyamatos
#define ADC_START            ADCSRA= 0B11001000+ ADC_PRESC           // ADC inditas IT-hez

#define ADC_DISABLE          B_0( ADCSRA, ADEN)                      // ADC tiltasa
#define ADC_ENABLE           B_1( ADCSRA, ADEN)                      // ADC tiltas feloldasa




// TC0 szamlalohoz - 8 bit
// TCNT0, TCCR0, TIFR, TIMSK, SFIOR
// IT_TC0_OVF, amikor TCNT0 255-rol 0-ra ugrik.

#define TC0_PRESCALER_RESET  B_1( SFIOR, PSR10)                        // TC0/TC1 elooszto reset
#define TC0_RESET            TCNT0= 0
#define TC0_STOP             TCCR0&= 0B11111000                        // nincs orajel
#define TC0_F_1              TCCR0&= 0B11111000; TCCR0|= 0B00000001    // F_CPU /    1
#define TC0_F_8              TCCR0&= 0B11111000; TCCR0|= 0B00000010    // F_CPU /    8
#define TC0_F_64             TCCR0&= 0B11111000; TCCR0|= 0B00000011    // F_CPU /   64
#define TC0_F_256            TCCR0&= 0B11111000; TCCR0|= 0B00000100    // F_CPU /  256
#define TC0_F_1024           TCCR0&= 0B11111000; TCCR0|= 0B00000101    // F_CPU / 1024
#define TC0_EXT_FALLING      TCCR0&= 0B11111000; TCCR0|= 0B00000110    // T0 lefuto el
#define TC0_EXT_RISING                           TCCR0|= 0B00000111    // T0 felfuto el

// szamol
#define TC0_COUNT             ""




// TC1 szamlalohoz - 16 bit
// TCNT1, ICR1, OCR1A, OCR1B, TCCR1A, TCCR1B, TIFR, TIMSK, SFIOR
// Szamlalo ujraallitasa elott TCCR1A-t es TCCR1B-t torolni kell!
// CLOCK - f_hz= ( F_CPU/ osztas)/ ( OCR1_+ 1)
// WAVE  - f_hz= F_CPU / ( 2* osztas *( OCR1_+ 1)
// TIMER - ido_usec= (OCR1- 1/ ( F_CPU/ osztas))* 1000000
// PWM8  - f_hz= F_CPU/ osztas/ 256
// PWM10 - f_hz= F_CPU/ osztas/ 1024

// TCR1B  TCR1B  TCR1A  TCR1A
// WGM13  WGM12  WGM11  WGM10
//     0      0      0      0 - Normal
//     0      1      0      0 - CTC
//     0      1      0      1 - Fast PWM 8
//     0      1      1      1 - FAST PWM 10

//  TCR1A  TCR1A
// COM1_1 COM1_0
//      0      0 - NON PWM DISCONNECT
//      0      1 - NON PWM TOGGLE
//      1      1 - PWM

#define TC1_PRESCALER_RESET   B_1( SFIOR, PSR10)                          // TC0/TC1 elooszto reset
#define TC1_RESET             TCNT1= 0
#define TC1_STOP              TCCR1B&= 0B11111000                         // nincs orajel
#define TC1_F_1               TCCR1B&= 0B11111000; TCCR1B|= 0B00000001    // F_CPU /    1
#define TC1_F_8               TCCR1B&= 0B11111000; TCCR1B|= 0B00000010    // F_CPU /    8
#define TC1_F_64              TCCR1B&= 0B11111000; TCCR1B|= 0B00000011    // F_CPU /   64
#define TC1_F_256             TCCR1B&= 0B11111000; TCCR1B|= 0B00000100    // F_CPU /  256
#define TC1_F_1024            TCCR1B&= 0B11111000; TCCR1B|= 0B00000101    // F_CPU / 1024
#define TC1_EXT_FALLING       TCCR1B&= 0B11111000; TCCR1B|= 0B00000110    // T1 lefuto el
#define TC1_EXT_RISING                             TCCR1B|= 0B00000111    // T1 felfuto el
#define TC1_CTC               B_1( TCCR1B, WGM12)
#define TC1_PWM8              B_1( TCCR1B, WGM12); B_1( TCCR1A, WGM10)
#define TC1_PWM10             B_1( TCCR1B, WGM12); B_1( TCCR1A, WGM11); B_1( TCCR1A, WGM10)

// szamol
#define TC1_COUNT       ""

// OCR1A erteknel aktivalja IT_TC1_COMPA-t
#define TC1_TIMER_A                         IT_TC1_COMPA_ENABLE
#define TC1_TIMER_A_OCR( ido_usec, osztas)  OCR1A= OCR_TIMER( ido_usec, osztas)

// OCR1B erteknel IT_TC1_COMPB-t aktivalja
#define TC1_TIMER_B                         IT_TC1_COMPB_ENABLE
#define TC1_TIMER_B_OCR( ido_usec, osztas)  OCR1B= OCR_TIMER( ido_usec, osztas)

// OCR1A erteknel szamlalot nullazza es invertalja OC1A labat
#define TC1_WAVE_A                          TC1_CTC; B_1( TCCR1A, COM1A0); PB1_OUT
#define TC1_WAVE_A_OCR( f_hz, osztas)       OCR1A= OCR_WAVE( f_hz, osztas)

// OCR1B erteknel szamlalot nullazza es invertalja OC1B labat
#define TC1_WAVE_B                          TC1_CTC; B_1( TCCR1A, COM1B0); PB2_OUT
#define TC1_WAVE_B_OCR( f_hz, osztas)       OCR1B= OCR_WAVE( f_hz, osztas)

// OCR1A erteknel szamlalot nullaza es IT_TC1_COMPA-t aktivalja
#define TC1_CLOCK_A                         TC1_CTC; IT_TC1_COMPA_ENABLE
#define TC1_CLOCK_A_OCR( f_hz, osztas)      OCR1A= OCR_CLOCK( f_hz, osztas)

// OCR1B erteknel szamlalot nullazza es IT_TC1_COMPB-t aktivalja
#define TC1_CLOCK_B                         TC1_CTC; IT_TC1_COMPB_ENABLE
#define TC1_CLOCK_B_OCR( f_hz, osztas)      OCR1B= OCR_CLOCK( f_hz, osztas)

// OCR1A erteknel OC1A lab= 1, majd 0xFF erteknel OC1A lab= 0
#define TC1_PWM8_A                          TC1_PWM8; B_1( TCCR1A, COM1A1); B_1( TCCR1A, COM1A0); PB1_OUT

// OCR1B erteknel OC1B lab= 1, majd 0xFF erteknel OC1B lab= 0
#define TC1_PWM8_B                          TC1_PWM8; B_1( TCCR1A, COM1B1); B_1( TCCR1A, COM1B0); PB2_OUT

// OCR1A erteknel OC1A lab= 1, majd 0x03FF erteknel OC1A lab= 0
#define TC1_PWM10_A                         TC1_PWM10; B_1( TCCR1A, COM1A1); B_1( TCCR1A, COM1A0); PB1_OUT

// OCR1B erteknel OC1B lab= 1, majd 0x03FF erteknel OC1B lab= 0
#define TC1_PWM10_B                         TC1_PWM10; B_1( TCCR1A, COM1B1); B_1( TCCR1A, COM1B0); PB2_OUT




// TC2 szamlalohoz - 8 bit
// TCNT2, OCR2, TCCR2, TIFR, TIMSK, SFIOR
// OCR2= TOP
// IT_TC2_OVF, amikor TCNT2 255-rol 0-ra ugrik.
// IT_TC2_COMP, amikor TCNT2 == OCR2
// Szamlalo ujraallitasa elott TCCR2-t torolni kell!
// TIMER - ido_usec= (OCR1- 1/ ( F_CPU/ osztas))* 1000000
// CLOCK - f_hz= ( F_CPU/ osztas)/ ( OCR2+ 1)
// WAVE  - f_hz= F_CPU / ( 2* osztas *( OCR2+ 1)
// PWM   - f_hz= F_CPU/ osztas/ 256

//  TCCR2  TCCR2
//  WGM21  WGM20
//      0      0  - Normal
//      1      0  - CTC
//      1      1  - Fast PWM

//  TCCR2  TCCR2
//  COM21  COM20
//      0      0  - NON PWM DISCONNECT
//      0      1  - NON PWM TOGGLE
//      1      1  - PWM

#define TC2_PRESCALER_RESET   B_1( SFIOR, PSR2)                        // TC2 elooszto reset
#define TC2_RESET            TCNT2= 0
#define TC2_STOP             TCCR2&= 0B11111000                        // nincs orajel
#define TC2_F_1              TCCR2&= 0B11111000; TCCR2|= 0B00000001    // F_CPU /    1
#define TC2_F_8              TCCR2&= 0B11111000; TCCR2|= 0B00000010    // F_CPU /    8
#define TC2_F_32             TCCR2&= 0B11111000; TCCR2|= 0B00000011    // F_CPU /   32
#define TC2_F_64             TCCR2&= 0B11111000; TCCR2|= 0B00000100    // F_CPU /   64
#define TC2_F_128            TCCR2&= 0B11111000; TCCR2|= 0B00000101    // F_CPU /  128
#define TC2_F_256            TCCR2&= 0B11111000; TCCR2|= 0B00000110    // F_CPU /  256
#define TC2_F_1024                               TCCR2|= 0B00000111    // F_CPU / 1024
#define TC2_CTC              B_1( TCCR2, WGM21)
#define TC2_PWM_             B_1( TCCR2, WGM21); B_1( TCCR2, WGM20)

// szamol
#define TC2_COUNT             ""

// OCR2 ertekenel IT_TC2_COMP-t aktivalja
#define TC2_TIMER                         IT_TC2_COMP_ENABLE
#define TC2_TIMER_OCR( ido_usec, osztas)  OCR2= OCR_TIMER( ido_usec, osztas)

// OCR2 ertekenel szamlalot nullazza es IT_TC2_COMP-t aktivalja
#define TC2_CLOCK                         TC2_CTC; IT_TC2_COMP_ENABLE
#define TC2_CLOCK_OCR( f_hz, osztas)      OCR2= OCR_CLOCK( f_hz, osztas)

// OCR2 ertekenel szamlalot nullazza es OC2 labat invertalja
#define TC2_WAVE                          TC2_CTC; B_1(TCCR2,COM20); PB3_OUT
#define TC2_WAVE_OCR( f_hz, osztas)       OCR2= OCR_WAVE( f_hz, osztas)

// OCR2 ertekenel OC2 lab= 1, majd 0xFF erteknel OC2 lab= 0
#define TC2_PWM                           TC2_PWM_; B_1(TCCR2,COM21); B_1(TCCR2,COM20); PB3_OUT
